/**
 * Title:       Price request sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   PriceSenderWithReply.java
 * Created:     December 11, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a Control queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api;

import javax.jms.*;


public class PriceSenderWithReply extends AbstractQueueSenderWithReply
{
    public PriceSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send a Control message, waiting for Price message reply
    public Price sendControlGetReply(Control control) throws JMSException, Exception
    {
        // Construct Control request message by supplying
        // the number of days back from today or a date, 0 (zero) means the current date
        // and the symbol of the financial instrument for which the price is requested
        // control.setControlType(Enums.ControlType.PRICE_REQUEST);
        // control.setRequestText(Integer.toString(noOfDays));
        // or control.setRequestText(Date.toString(date));
        // control.setReplyText(String Symbol);

        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(control);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.CONTROL.toString());
        message.setStringProperty("CTRL_TYPE", Enums.ControlType.PRICE_REQUEST.toString());
        message.setStringProperty("USER_ID", control.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create Control sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process Price reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.PRICE.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(control.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    Price replyPrice = (Price)replyObject.getObject();

                    replyReceiver.close();
                    requestSender.close();
                    
                    return replyPrice;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        replyReceiver.close();
        requestSender.close();
                    
        return null;
    }
}

