/**
 * Title:       User queue sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   UserSenderWithReply.java
 * Created:     November 30, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an User queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api;

import javax.jms.*;


public class UserSenderWithReply extends AbstractQueueSenderWithReply
{
    public UserSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send an User data request message, waiting for reply
    public User sendUserGetReply(User user) throws JMSException, Exception
    {
        // Construct User request message
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(user);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.USER.toString());
        message.setStringProperty("CTRL_TYPE", Enums.ControlType.USER_REQUEST.toString());
        message.setStringProperty("USER_ID", user.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create User sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process User reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.USER.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(user.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    User replyUser = (User)replyObject.getObject();

                    replyReceiver.close();
                    requestSender.close();
                    
                    return replyUser;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        replyReceiver.close();
        requestSender.close();
                    
        return null;
    }

    // Create and send an User Add message, waiting for reply
    public User sendUserAddGetReply(User user) throws JMSException, Exception
    {
        // Construct User request message
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(user);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.USER.toString());
        message.setStringProperty("CTRL_TYPE", Enums.ControlType.USER_ADD_REQUEST.toString());
        message.setStringProperty("USER_ID", user.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create User sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process User reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.USER.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(user.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    User replyUser = (User)replyObject.getObject();

                    replyReceiver.close();
                    requestSender.close();
                    
                    return replyUser;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        replyReceiver.close();
        requestSender.close();
                    
        return null;
    }

    // Create and send an User Update message, waiting for reply
    public User sendUserUpdateGetReply(User user) throws JMSException, Exception
    {
        // Construct User request message
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(user);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.USER.toString());
        message.setStringProperty("CTRL_TYPE", Enums.ControlType.USER_UPDATE_REQUEST.toString());
        message.setStringProperty("USER_ID", user.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create User sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process User reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.USER.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(user.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    User replyUser = (User)replyObject.getObject();

                    replyReceiver.close();
                    requestSender.close();
                    
                    return replyUser;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        replyReceiver.close();
        requestSender.close();
                    
        return null;
    }
}

